-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local List = require "engine.ui.List"
local GetQuantity = require "engine.dialogs.GetQuantity"

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
	self:generateList()
	Dialog.init(self, "Create Object", 1, 1)

	local list = List.new{width=400, height=500, list=self.list, fct=function(item) self:use(item) end}

	self:loadUI{
		{left=0, top=0, ui=list},
	}
	self:setupUI(true, true)

	self.key:addCommands{ __TEXTINPUT = function(c)
		for i = list.sel + 1, #self.list do
			local v = self.list[i]
			if v.name:sub(1, 1):lower() == c:lower() then list:select(i) return end
		end
		for i = 1, list.sel do
			local v = self.list[i]
			if v.name:sub(1, 1):lower() == c:lower() then list:select(i) return end
		end
	end}
	self.key:addBinds{ EXIT = function() game:unregisterDialog(self) end, }
end

function _M:use(item)
	if not item then return end
	game:unregisterDialog(self)

	if item.unique then
		local n = game.zone:finishEntity(game.level, "object", item.e)
		n:identify(true)
		game.zone:addEntity(game.level, n, "object", game.player.x, game.player.y)
	else
		game:registerDialog(GetQuantity.new("Number of items to make", "1-100", 20, 100, function(qty)
			Dialog:yesnoPopup("Ego", "Make an ego item if possible?", function(ret)
				if not ret then
					for i = 1, qty do
						local n = game.zone:finishEntity(game.level, "object", item.e, {ego_chance=-1000})
						n:identify(true)
						game.zone:addEntity(game.level, n, "object", game.player.x, game.player.y)
					end
				else
					Dialog:yesnoPopup("Greater Ego", "Make an ego item if possible?", function(ret)
						local f
						if not ret then
							f = {ego_chance=1000}
						else
							f = {ego_chance=1000, properties={"greater_ego"}}
						end

						for i = 1, qty do
							local n = game.zone:finishEntity(game.level, "object", item.e, f)
							n:identify(true)
							game.zone:addEntity(game.level, n, "object", game.player.x, game.player.y)
						end
					end)
				end
			end)
		end), 1)
	end
end

function _M:generateList()
	local list = {}

	for i, e in ipairs(game.zone.object_list) do
		if e.name and e.rarity then
			list[#list+1] = {name=e.name, unique=e.unique, e=e}
		end
	end
	table.sort(list, function(a,b)
		if a.unique and not b.unique then return true
		elseif not a.unique and b.unique then return false end
		return a.name < b.name
	end)

	local chars = {}
	for i, v in ipairs(list) do
		v.name = v.name
		chars[self:makeKeyChar(i)] = i
	end
	list.chars = chars

	self.list = list
end
